#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require 'zk'
require 'getopt/std'
require 'tempfile'

class IncidentGroupReportsController < ApplicationController
  include Exportable

  ## ======================================================================
  # => INCLUDES
  ## ======================================================================

  include AlarmsHelper
  include Redborder::Filters::Storage::Session
  include IncidentSetters


  helper GroverHelper::GroverAssetsHelper

  ## ======================================================================
  # => Controller properties
  ## ======================================================================

  respond_to :html
  respond_to :js, :json,
             only: [:create, :update, :destroy, :update]

  # layout false, only: [:import, :send_now, :new_user, :new_sensor, :generate,
  #                      :remove_page, :remove_page_modal, :sensor_childrens]

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_report,
                only: [:show, :edit, :destroy, :update]

  COLUMN_MAP = {
    "domain" => "sensor_id",
    "priority" => "priority_id",
    "status" => "incident_status_id",
    "user" => "user_id",
    "source" => "source",
  }

  def new 
    @incident_group_report = IncidentGroupReport.new
  end

  def create
    # Get all the params
    to_save = create_report_params

    # Default values
    to_save.merge!(
      number_pages: 1,
    )

    # Save the collaborators and the sensors
    collaborators = to_save.delete(:collaborators)
    sensors_collaborators = to_save.delete(:sensors_collaborators)

    # Create the Report
    @incident_group_report = IncidentGroupReport.new(to_save)

    # Associate the user
    @incident_group_report.owner = current_user

    Rails.logger.error "\nCREATE PARAM #{to_save}\n"
    if @incident_group_report.save
      save_collaborators @incident_group_report, collaborators
      save_sensors_collaborators @incident_group_report, sensors_collaborators
      flash.now[:notice] = I18n.t('incident_group_report.create.success')
      # Force changing the tab to Incident Reports
      redirect_to reports_path
    end
  end

  def edit
  end

  def update
     # Get all the params
    to_save = create_report_params

    # Default values
    to_save.merge!(
      number_pages: 1,
    )

    # Save the collaborators and the sensors
    collaborators = to_save.delete(:collaborators)
    sensors_collaborators = to_save.delete(:sensors_collaborators)
    
    if @incident_group_report.update to_save
      save_collaborators @incident_group_report, collaborators
      save_sensors_collaborators @incident_group_report, sensors_collaborators
      flash.now[:notice] = I18n.t('incident_group_report.create.success')
      # Force changing the tab to Incident Reports
      redirect_to reports_path
    end
  end

  def destroy
    if @incident_group_report.owner == current_user
      @incident_group_report.destroy
    # else
    #   @report.report_users.find_by(user_id: current_user.id).try(:destroy)
    end

    respond_with @incident_group_report, location: reports_url
  end

  def show
  respond_to do |format|
    format.pdf do
      Rails.logger.error "\nLlamaando a generar pdf\n"
      Rails.logger.error "\nID: #{@incident_group_report.id} Con filtros: #{@incident_group_report.condition}"
      @incidents = filter_by_conditions

      Rails.logger.error "\nIncident count: #{@incidents.count}\n\nIDS: #{@incidents.pluck(:id)}\n\n"
      
      @incidents.each do | incident |
        pdf_data = incident.incident_reports.download_report.to_pdf
        Tempfile.open(['temp', '.pdf']) do |file|
          file.binmode
          file.write(pdf_data)
          file.rewind
          # create_incident_historical(@incident_report.id, file)
        end
        
        send_data pdf_data, filename: "#{@report_name}" + '.pdf',
        type: 'application/pdf',
        disposition: 'inline'
        break
      end

      end
    end
  end
  
  def filter_by_conditions

    set_sensor_names
    @incidents = Incident.where(visible: true, sensor_id: @sensor_names.keys)
    JSON.parse(@incident_group_report.condition).each do |_, rule|
      if rule["enabled"]
        column = COLUMN_MAP[rule["column"]]
        operator = ''
        case rule["operator"]
        when 'is'
          operator = ' = '
        when 'is_not'
          operator = ' != '
        end
        value = rule["value"]
        case rule["column"]
        when 'user'
          Rails.logger.error "\nincident_users.user_id #{operator} #{rule["value"]}\n"
          # If there is no assigned users and the operator is not, return current incidents
          # instead of nothing
          next if @incidents.joins(:incident_users).count == 0 and rule["operator"] == 'is_not'
          @incidents = @incidents.joins(:incident_users).where("incident_users.user_id #{operator} #{rule["value"]}")
          next
        when 'source'
          value = "'#{rule["value"]}'"
        end
        filter = "incidents.#{column} #{operator} #{value}"
        @incidents = @incidents.where(filter)
      end
    end
    @incidents
  end

  def create_report_params
    schedule_params
    params.require(:incident_group_report)
      .permit(:name, :description,
              :schedule_type, :email_list, :schedule_days, :schedule_hour,
              :condition,
              collaborators: [:user_id, :can_edit],
              sensors_collaborators: [:sensor_id, :can_edit])
  end

  def schedule_params
    hour = params[:incident_group_report_hour]
    minute = params[:incident_group_report_minute]
    params[:incident_group_report][:schedule_hour] = "#{hour}:#{minute}"
  end

  def save_collaborators(report, collaborators)
    # Remove all relations, except those with the owner and the current user
    report.report_users.each do |x|
      x.delete unless [x.incident_group_report.user_id, current_user.id].include? x.user_id
    end

    return if collaborators.nil?

    # Dont save relations with the owner or the current user
    collaborators.each do |user|
      next if user['user_id'] == current_user.id ||
              user['user_id'] == report.user_id

      report.report_users.create(
        user_id: user['user_id'],
        can_edit: (user['can_edit'] == 'on')
      )
    end
  end

  def save_sensors_collaborators(report, sensors_collaborators)
    # Remove all relations
    report.report_sensors.destroy_all
    return if sensors_collaborators.nil?

    # Save all relations
    sensors_collaborators.each do |_index, sensor|
      next unless [:all, :admin, :none].include? sensor['can_edit'].to_sym

      report.report_sensors.create(
        sensor_id: sensor['sensor_id'],
        can_edit: sensor['can_edit']
      )
    end
  end

  private

  def set_report
    @incident_group_report = IncidentGroupReport.find(params[:id])
  end

end