#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module ReportsHelper
  def report_block_get_category(category)
    if category == "cpu"
      category.upcase
    else
      category.capitalize
    end
  end

  #
  # Generate the options for move page select when delete a page
  #
  # == Parameters:
  # report::
  #   Report to delete the page
  # page::
  #   Integer of page to delete
  #
  # == Returns:
  # An Array with format: ['Page X', X].
  #
  def move_pages(report, page)
    options = []
    for i in 1..report.number_pages do
      next if page.to_i == i
      options << [ I18n.t('reports_helper.page') + " " + i.to_s , i]
    end
    # Return options
    options
  end

  #
  # Set the HTML code for the monthly days selector
  #
 

  #
  # Return the name of the tab for redborder objects
  #
  # == Parameters:
  # tab::
  #   Symbol of the tab
  #
  # == Returns:
  # String with the name
  #
  def report_tab_name(tab)
    I18n.t("reports.index.#{tab}")
  end

  #
  # Create the tabs to navigate between report types
  #
  # == Parameters:
  # tabs::
  #   Array of symbols with tabs to show
  #
  # == Returns:
  # Safe string with HTML code
  #
  def report_tabs(tabs)
    html = ''
    tabs.each do |tab|
      html +=
        content_tag(:li, class: (session[:report] == tab ? 'active' : ''),
                         id: tab) do
          link_to(report_tab_name(tab), "##{tab}", :'data-toggle' => 'tab')
        end
    end
    # Return HTML safe string
    html.html_safe
  end
end
