# app/services/incident_report_generator.rb
class IncidentReportGenerator

  include IncidentSetters

  def initialize(incident)
    @incident = incident
  end

  def generate_pdf
    set_elements
    load_actions_and_worklog_data
    calculate_nodes
    Rails.logger.error "\nGenerating pdf for incident #{@incident.id}\n"

    @time = Time.now
    @time_week_ago = Time.now - 7.days
    #@time_month_ago = Time.now - 1.month
    @indicators = load_indicators(@elements)

    assigned_users_ids = IncidentUser.where(incident_id: @incident.id).pluck(:user_id)
    @assigned_users_info = User.where(id: assigned_users_ids).pluck(:name, :email).to_h

    aux_org_name = Sensor.where(id:@incident.sensor_id).pluck(:property)
    first_org_name = aux_org_name[0]['path'].split('/').first

    @org_name = first_org_name.to_s.strip.empty? ? '' : "-#{first_org_name}"

    start_date = @time_week_ago.strftime("%d-%m-%y")
    end_date = @time.strftime("%d-%m-%y")

    @report_name = 'RB' + "#{start_date}" + '/' + "#{end_date}" + "#{@org_name}"

    @incident_report = @incident.incident_reports

    # create an instance of ActionView, so we can use the render method outside of a controller
    av = ActionView::Base.new
    av.view_paths = ActionController::Base.view_paths
    # need these in case your view constructs any links or references any helper methods.
    av.class_eval do
      include Rails.application.routes.url_helpers
      include ApplicationHelper
      include DashboardHelper
      include WidgetsHelper
      include ReportBlocksHelper
      include ManagersHelper
      include IncidentsHelper
    end

    redborder_config = YAML.load_file(Rails.root.join('config/redborder_config.yml'))[Rails.env].symbolize_keys
    Rails.application.routes.default_url_options = {host: redborder_config[:url]}

    html_content = av.render(template: 'incidents/incident_report.pdf.erb', 
    locals: { 
    incident: @incident,
    incident_report: @incident_report,
    time: @time,
    report_name: @report_name,
    assigned_users_info: @assigned_users_info,
    indicators: @indicators,
    observables_by_type: @observables_by_type,
    observables: @observables,
    actions_taken_data: @actions_taken_data,
    assets: @assets
    })
    # locals: { incident_report: @incident_report },
    footer_html = av.render(partial: 'incidents/incident_report_footer', formats: [:html])



    # 1 lin = 96px
    grover = Grover.new(
      html_content,
      display_header_footer: true,
      header_template: '<div></div>',
      footer_template: footer_html,
      orientation: "Portrait",
      margin: { top: 24, left: 48, right: '1in', bottom: '1in' },
      format: "A4",
      scale: 0.8,
      print_backgroud: true
    )
    return grover
    # # Generate the PDF data
    # pdf_data = grover.to_pdf

    # Tempfile.open(['temp', '.pdf']) do |file|
    #   file.binmode
    #   file.write(pdf_data)
    #   file.rewind
    #   create_incident_historical(@incident_report.id, file)
    # end

    # send_data pdf_data, filename: "#{@report_name}" + '.pdf',
    # type: 'application/pdf',
    # disposition: 'inline'

  end

end